import { findCurrentTaskMarkResult } from "@/components/mark/store";
import { httpApp } from "@/plugins/axiosApp";
import { Setting, UISetting } from "@/types";

/** 清除评卷任务（之前锁住的任务之类的） */
export async function clearMarkTask() {
  return httpApp.post("/mark/clear");
}

/** 获取评卷设置 */
export async function getSetting() {
  return httpApp.post("/mark/getSetting");
}

/** 获取评卷状态 */
export async function getStatus() {
  return httpApp.post("/mark/getStatus");
}

/** 获取评卷分组 */
export async function getGroup() {
  return httpApp.post("/mark/getGroup");
}

/** 获取评卷任务 */
export async function getTask() {
  return httpApp.post("/mark/getTask");
}

/** 更新评卷UI */
export async function updateUISetting(
  mode?: Setting["mode"],
  uiSetting?: UISetting
) {
  const form = new FormData();
  uiSetting && form.append("uiSetting", JSON.stringify(uiSetting));
  mode && form.append("mode", mode);
  return httpApp.post("/mark/updateSetting", form);
}

/** 获取评卷历史任务 */
export async function getHistoryTask({
  pageNumber = 1,
  pageSize = 10,
  order = "markerTime",
  sort = "DESC",
  secretNumber = null,
}: {
  pageNumber?: number; // 从1开始
  pageSize?: number;
  order?: "markerTime" | "markerScore";
  sort?: "ASC" | "DESC";
  secretNumber?: string | null;
}) {
  const form = new FormData();
  form.append("pageNumber", pageNumber + "");
  form.append("pageSize", pageSize + "");
  form.append("order", order);
  form.append("sort", sort);
  secretNumber && form.append("secretNumber", secretNumber);
  return httpApp.post("/mark/getHistory", form);
}

/** 保存评卷任务 */
export async function saveTask() {
  const markResult = findCurrentTaskMarkResult();
  if (markResult) {
    markResult.specialTagList = [];
    markResult.problem = false;
    markResult.spent = Date.now() - markResult.spent;

    return httpApp.post("/mark/saveTask", markResult);
  }
}

/** 获取分组列表 */
export async function getGroups() {
  return httpApp.post("/mark/getGroup");
}

/** 切换分组 */
export async function switchGroup(markerId: number) {
  const form = new FormData();
  form.append("markerId", markerId + "");
  return httpApp.post("/mark/subjectSelect", form);
}

/** 获取用户信息 */
export async function changeUserInfo(name: string, password?: string) {
  const form = new FormData();
  form.append("name", name);
  password && form.append("password", password);
  return httpApp.post("/mark/changeName", form);
}
